# -*- coding: utf-8 -*-
#!/usr/bin/python3

import copy
import numpy as np
from numpy import sin, cos, exp, sqrt
from numpy.linalg import solve
import os
import os.path
import re
import scipy.constants as physics
import shutil
import subprocess
import time
from scipy.optimize import curve_fit


###################################################################################
#
#  計算用のインプットファイルを読み込むためのクラス
#
###################################################################################
class SUPERFISH(object):
    program = 'SUPERFISH'

    # 計算結果 2a [mm], 2b [mm], D [mm], beta [1/c],
    #         facc [MHz], Rs [M Ohm/m], Q [dim less], vg [1/c], k [V/(C Cell)]
    #         RF [W], U [J]
    
    cell = {'2a':np.nan, '2b':np.nan, 'D':np.nan, 't':np.nan, 'beta':np.nan,\
            'facc':np.nan, 'Q':np.nan, 'Rs':np.nan, 'r_ov_Q':np.nan, 'vg':np.nan,\
            'k':np.nan,\
            'facc_SS':np.nan, 'U_SS':np.nan, 'Q_SS':np.nan, 'Rs_SS':np.nan,\
            'Ez_S_SS':np.nan, 'Ez_E_SS':np.nan,\
            'facc_OO':np.nan, 'U_OO':np.nan, 'Q_OO':np.nan, 'Rs_OO':np.nan,\
            'Ez_S_OO':np.nan, 'Ez_E_OO':np.nan,\
            'f0':np.nan, 'f1':np.nan, 'f2':np.nan, 'f3':np.nan,\
            'f4':np.nan, 'f5':np.nan, 'f6':np.nan,\
            'kappa':np.nan, 'fa':np.nan, 'disp_vg':np.nan,\
            'alpha':np.nan, 'RF':np.nan,\
            'gdf_SS':'', 'gdf_OO':'',\
            'zs':np.nan, 'ze':np.nan, 'RF':np.nan, 'U':np.nan, 'phase':np.nan,\
            'ampSS':np.nan, 'ampOO':np.nan}
    
    cpl = {'p_len':np.nan, 'p_dia':np.nan, 'ra':np.nan, 'D_half':np.nan, \
           '2b':np.nan, '2a':np.nan,\
           'freq':np.nan, 'U':np.nan, 'Q':np.nan, 'Rs':np.nan, 'ro_v_Q':np.nan,\
           'Rs':np.nan, 'Ez_S':np.nan, 'Ez_E':np.nan,\
           'gdf':'',\
           'zs':np.nan, 'ze':np.nan, 'phase':np.nan,\
           'amp':np.nan}
        
    # -------------------------------------------------------------------
    # SUPERFISH で自動計算する加速管の定義ファイルの読込み
    #     title: SUPERFISH の計算のタイトル
    #     dx   : メッシュサイズ [mm]
    #     freq : ターゲットの加速周波数 [MHz]
    #     cell : 加速セルの形状データ
    #     GPT  : gdf ファイルと作るときのデータ
    # -------------------------------------------------------------------
    def __init__(self, title, dx, freq, cell, GPT):
        ''' コンストラクター: 初期化を行う．'''
        self.title = title
        self.dx = dx
        self.facc = freq
        self.input_cell = cell
        self.GPT = GPT

        self.lmd = physics.c/(self.facc*1e6)*1e3    # Wave length [mm]
        self.N_cell = len(self.input_cell)
        self.tw_cell={}
        for i in range(1, self.N_cell+1):
            self.tw_cell[i]= copy.deepcopy(SUPERFISH.cell)
            self.tw_cell[i]['beta'] = self.input_cell[i]['beta']
            self.tw_cell[i]['2a']   = self.input_cell[i]['2a']
            self.tw_cell[i]['t']    = self.input_cell[i]['t']
            self.tw_cell[i]['D']    = self.input_cell[i]['beta']*self.lmd/3.0

        self.af_dir      = '../superfish_af'
        self.af_dir_15   = self.af_dir+'/cell_15'
        self.af_dir_30   = self.af_dir+'/cell_30'
        self.af_dir_sw   = self.af_dir+'/sw'
        self.sf_file     = 'CELL'
        self.working_dir = '../working_dir'
        self.report_dir  = '../results'
        self.GPT_dir     = '../GPT'
        self.home_dir    = os.getcwd()

        self.it_max     = 10              # 2b 調整時の最大イタレーション番号
        self.error      = 1e-6            # 2b 調整時の最大エラーの許容値
        self.seg_Wall_L = 5               # Q を計算する左端のセグメント番号
        self.seg_Wall_R = 16              # Q を計算する右端のセグメント番号
        self.N_divR     = 1000            # ポインティングベクトル計算の分割数
        self.zero_pi_freq = (2750.0, 2930.0)  # 0, pi モード周波数を測定するときの範囲 (初期値)
        self.ndisp      = 50              # scan で共振周波数を求める場合の周波数分割

        print('Title: ', self.title)
        print('Mesh Interval [mm]: ', self.dx)
        print('Acc Frequency [MHz]: ', self.facc)
        print('Wave length [mm]: ', self.lmd)

        if os.path.exists(self.af_dir_15):   shutil.rmtree(self.af_dir_15)
        if os.path.exists(self.af_dir_30):   shutil.rmtree(self.af_dir_30)
        if os.path.exists(self.af_dir_sw):   shutil.rmtree(self.af_dir_sw)
        if os.path.exists(self.af_dir):      shutil.rmtree(self.af_dir)
        if os.path.exists(self.GPT_dir):     shutil.rmtree(self.GPT_dir)
        if os.path.exists(self.working_dir): shutil.rmtree(self.working_dir)
        if not os.path.exists(self.report_dir):os.mkdir(self.report_dir)
        
        os.mkdir(self.af_dir)
        os.mkdir(self.af_dir_15)
        os.mkdir(self.af_dir_30)
        os.mkdir(self.af_dir_sw)
        os.mkdir(self.GPT_dir)
        os.mkdir(self.working_dir)
        os.chdir(self.working_dir)

        

    # -------------------------------------------------------------------
    #  accelerator paramters as a function of 2a [mm]  ref: EM-10008A
    #     x: 2a [mm]
    #     return:
    #        fn_2b [mm]   cavity diameter (2b)
    # -------------------------------------------------------------------
    def fn_2b(self, x):
        lmd = physics.c/(self.facc*1.0E6)
        k = 2*np.pi/lmd
        b = 2.4048/k*1E3
        return 2*b
        
    # -------------------------------------------------------------------
    #  tunning the accelerating cavity radius. all cell
    # -------------------------------------------------------------------
    def all_cell_2b_tune(self):
        ''' すべての加速せるの 2b を調整する．'''
        for i in range(1, self.N_cell+1):
            self.tune_2b_acc(i)

            
    # -------------------------------------------------------------------
    # SUPERFISH の実行
    #   is_disp:  True: 分散関係を計算する      False: 計算しない
    #   is_GPT:   True: gdf ファイルを作成する  False: 作成しない
    # -------------------------------------------------------------------
    def all_cell_parameter_calculation(self, is_disp, is_GPT):
        ''' すべての加速せるのパラメーターを計算する '''
        self.is_disp = is_disp
        self.is_GPT  = is_GPT
        for i in range(1, self.N_cell+1):
            print('\ncell paramter calculation ({0:d}/{1:d})'.\
                  format(i, self.N_cell))
            print('2a: {0:.3f}'.format(self.tw_cell[i]['2a']))
            self.cal_SS_OO_mode(i)
            if self.is_disp:
                self.cal_dispersion(cell_no=i)

        
    # -------------------------------------------------------------------
    #  tunning the accelerating cavity radius
    #    cell_no:  周波数調整するセルの番号
    # -------------------------------------------------------------------
    def tune_2b_acc(self, cell_no):
        ''' 加速モードの周波数を調整する．'''
        cell_2a = self.tw_cell[cell_no]['2a']
        tune_2b_acc = []
        sf_frequency = []
        tune_2b_acc.append(self.fn_2b(cell_2a))    # 初期値
        print('\nTune Cell: {0:d}'.format(cell_no))
        print('2a: {0:.3f}'.format(cell_2a))
            
        for j in range(0, self.it_max+1):
            self.tw_cell[cell_no]['2b']= tune_2b_acc[j]
            self.mk_file_af(self.tw_cell[cell_no], ('H', 'F'), 0.0, 0, 0, self.facc)
            self.run_autofish()
            sf_frequency.append(self.read_frequency())
            print('\t{0:d}\t{1:.6f}\t{2:.3f}'.\
                  format(j, tune_2b_acc[j], sf_frequency[j]))
            if np.abs((sf_frequency[j]-self.facc)/self.facc) < self.error:
                break
            if j==0:
                tune_2b_acc.append(sf_frequency[0]*tune_2b_acc[0]/self.facc)
            else:
                tune_2b_acc.append(\
                                   (tune_2b_acc[j]-tune_2b_acc[j-1])\
		                   /(sf_frequency[j]-sf_frequency[j-1])\
		                   *(self.facc-sf_frequency[j])\
		                   +tune_2b_acc[j])

    # -------------------------------------------------------------------
    #  calculate SS and OO modes
    #     cell_no: セル番号
    # -------------------------------------------------------------------
    def cal_SS_OO_mode(self, cell_no):
        ''' 二つの境界条件で加速モードを計算する． '''
        self.mk_seg(self.seg_Wall_L, self.seg_Wall_R)
        i = cell_no

        a  = self.tw_cell[i]['2a']/2.0
        b  = self.tw_cell[i]['2b']/2.0
        Lz = self.tw_cell[i]['D']*1.5;
    
        #----- calculate SS mode -----
        self.mk_file_af(self.tw_cell[i], ('H', 'F'), 0.0, 1, 1, self.facc)
        self.run_autofish()

        f_SS, U_SS, Q_SS, ZTT_SS, r_ov_Q_SS = self.read_SFO()
        Ez_S_SS, Ez_E_SS = self.side_Ez(0.0, self.tw_cell[i]['D']);
        rL_SS, HL_SS = self.set_field('Ht', 0.0, 0.0, b)
        rR_SS, HR_SS = self.set_field('Ht', Lz, 0.0, a)
        Rs_SS = ZTT_SS*2

        self.tw_cell[i]['facc_SS']  = f_SS
        self.tw_cell[i]['U_SS']     = U_SS
        self.tw_cell[i]['Q_SS']     = Q_SS
        self.tw_cell[i]['r_ovQ_SS'] = r_ov_Q_SS
        self.tw_cell[i]['Rs_SS']    = Rs_SS
        self.tw_cell[i]['Ez_S_SS']  = Ez_S_SS
        self.tw_cell[i]['Ez_E_SS']  = Ez_E_SS
        if self.is_GPT:
            gdf_file = 'cell_{0:02d}_SS.gdf'.format(i)
            self.tw_cell[i]['gdf_SS']   = gdf_file
            self.mk_gdf(gdf_file, self.tw_cell[i]['D'], self.tw_cell[i]['2a']/2.0+1.0)
        

        #----- calculate OO mode -----
        self.mk_file_af(self.tw_cell[i], ('H', 'F'), 0.0, 0, 0, self.facc)
        self.run_autofish()
        f_OO, U_OO, Q_OO, ZTT_OO, r_ov_Q_OO = self.read_SFO()
        Ez_S_OO, Ez_E_OO = self.side_Ez(0.0, self.tw_cell[i]['D']);
        rL_OO, EL_OO = self.set_field("Er", 0.0, 0.0, b)
        rR_OO, ER_OO = self.set_field("Er", Lz, 0.0, a)
        Rs_OO = ZTT_OO*2

        self.tw_cell[i]['facc_OO']  = f_OO
        self.tw_cell[i]['U_OO']     = U_OO
        self.tw_cell[i]['Q_OO']     = Q_OO
        self.tw_cell[i]['r_ovQ_OO'] = r_ov_Q_OO
        self.tw_cell[i]['Rs_OO']    = Rs_OO
        self.tw_cell[i]['Ez_S_OO']  = Ez_S_OO
        self.tw_cell[i]['Ez_E_OO']  = Ez_E_OO
        if self.is_GPT:
            gdf_file = 'cell_{0:02d}_OO.gdf'.format(i)
            self.tw_cell[i]['gdf_OO']   = gdf_file
            self.mk_gdf(gdf_file, self.tw_cell[i]['D'], self.tw_cell[i]['2a']/2.0+1.0)
        
        #------------- frequency check ------------
        f_error = np.abs(self.tw_cell[i]['facc_OO']-self.tw_cell[i]['facc_SS'])
        if 100*self.error < f_error/self.tw_cell[i]['facc_OO']:
            print('Frequency error !!\n')
            print('\tSS mode: {0:.6f}\n'.format(self.tw_cell[i]['facc_SS']))
            print('\tOO mode: {0:.6f}\n'.format(self.tw_cell[i]['facc_OO']))
            exit;

        U_tw, RF_pwr_L, RF_pwr_R = self.cal_RF_pwr(\
                                        U_SS, rL_SS, rR_SS, HL_SS, HR_SS,\
                                        U_OO, rL_OO, rR_OO, EL_OO, ER_OO)
        vg_L = np.abs(Lz*1e-3*RF_pwr_L/U_tw)
        vg_R = np.abs(Lz*1e-3*RF_pwr_R/U_tw)

        self.tw_cell[i]['facc']   = 0.5*(f_SS + f_OO)
        self.tw_cell[i]['Q']      = 0.5*(Q_SS + Q_OO)
        self.tw_cell[i]['Rs']     = 0.5*(Rs_SS + Rs_OO)   # [M Ohm/m]
        self.tw_cell[i]['r_ov_Q'] = 0.5*(r_ov_Q_SS + r_ov_Q_OO)*2/1.5  # 進行波(*2) セル当たり (/1.5)
        self.tw_cell[i]['vg']     = 0.5*(vg_L+vg_R)/physics.c

        omega = 2*np.pi*self.facc*1e6
        R     = self.tw_cell[i]['Rs']*1e6*self.tw_cell[i]['D']*1e-3   #[Ohm/cell]
        Q     = self.tw_cell[i]['Q']
        self.tw_cell[i]['k'] = omega*R/(4*Q)

        print('\n\t ---- Short - Short mode ----')
        print('\t Frequency    \t{0:.6f} [MHz]'.format(self.tw_cell[i]['facc_SS']))
        print('\t Stored Energy\t{0:.6e} [J]'.format(self.tw_cell[i]['U_SS']))
        print('\t Q value      \t{0:.2f}'.format(self.tw_cell[i]['Q_SS']))
        print('\t r/Q          \t{0:.3f} [Ohm]'.format(self.tw_cell[i]['r_ovQ_SS']))
        print('\t Rs           \t{0:.6f} [MOhm/m]'.format(self.tw_cell[i]['Rs_SS']))
        print('\t Ez(start)    \t{0:.6e}'.format(self.tw_cell[i]['Ez_S_SS']))
        print('\t Ez(end)      \t{0:.6e}'.format(self.tw_cell[i]['Ez_E_SS']))

        print('\n\t ---- Open - Open mode ----')
        print('\t Frequency    \t{0:.6f} [MHz]'.format(self.tw_cell[i]['facc_OO']))
        print('\t Stored Energy\t{0:.6e} [J]'.format(self.tw_cell[i]['U_OO']))
        print('\t Q value      \t{0:.2f} [MHz]'.format(self.tw_cell[i]['Q_OO']))
        print('\t r/Q          \t{0:.3f} [Ohm]'.format(self.tw_cell[i]['r_ovQ_OO']))
        print('\t Rs           \t{0:.6f} [MOhm/m]'.format(self.tw_cell[i]['Rs_OO']))
        print('\t Ez(start)    \t{0:.6e}'.format(self.tw_cell[i]['Ez_S_OO']))
        print('\t Ez(end)      \t{0:.6e}'.format(self.tw_cell[i]['Ez_E_OO']))

        print('\n\t ---- Group velocity calculation (left, right) ----')
        print('\tStoroed Energy in 1.5 cell [J]: {0:e} '.format(U_tw))
        print('\tRF power flow [W]:              {0:e}\t{1:e}'.\
              format(RF_pwr_L, RF_pwr_R))
        print('\tGrout velocity [m/sec]:         {0:e}\t{1:e}'.\
              format(vg_L, vg_R))
        print('\tvg/c:                           {0:e}\t{1:e}'.\
              format(vg_L/physics.c, vg_R/physics.c))

        file_path = self.af_dir_15+'/'+self.sf_file+'{0:02d}'.format(i)+'.af'
        os.replace(self.sf_file+'.af', file_path)


    # -------------------------------------------------------------------
    #  calculate dispersion relations
    # -------------------------------------------------------------------
    def cal_dispersion(self, cell_no):
        ''' 分散関係を計算する。 '''
        def disp(x, fa, kappa):
            return fa*sqrt(1-kappa*cos(x))
        
        print('\n\t ---- start dispersion calculaiton. ----')
        dxf = 0.3/self.dx
        
        # ----- 0, pi ラフな共振周波数を計算する -----
        self.mk_file_af(self.tw_cell[cell_no], ('H', 'H'), -0.5, 1, 1,\
                        (self.zero_pi_freq[0], self.zero_pi_freq[1], self.ndisp),
                        dx_fac=dxf)
        self.run_autofish()
        f_zero, f_pi = self.read_resonance_frequency_form_fcan()

        fs = f_zero - (f_pi - f_zero)*0.1   # 分散関係計算開始周波数
        fe = f_pi   + (f_pi - f_zero)*0.1   # 分散関係計算終了周波数
        freq = np.zeros(7)                  # n*pi/6 モードの周波数

        # ----- 0, 2pi/6, 4pi/6, pi モードのラフな共振周波数を計算する -----
        self.mk_file_af(self.tw_cell[cell_no], ('H', 'F', 'F', 'H'), -1.5, 1, 1,\
                        (fs, fe, self.ndisp), dx_fac=dxf)
        self.run_autofish()
        freq[0], freq[2], freq[4], freq[6] = self.read_resonance_frequency_form_fcan()

        # ----- pi/6, 3pi/6, 5pi/6 モードのラフな共振周波数を計算する -----
        self.mk_file_af(self.tw_cell[cell_no], ('H', 'F', 'F', 'H'), -1.5, 1, 0,\
                        (fs, fe, self.ndisp), dx_fac=dxf)
        self.run_autofish()
        freq[1], freq[3], freq[5] = self.read_resonance_frequency_form_fcan()

        # ----- 詳細計算 0, pi/6, 2pi/6 3pi/6, 4pi/6, 5pi/6, pi -----
        disp_freq  = np.zeros(7)
        disp_phase = np.array([i*np.pi/6.0 for i in range(0, 7)])
        for i in range(0, len(freq)):
            if i%2 == 0:
                self.mk_file_af(self.tw_cell[cell_no], ('H', 'F', 'F', 'H'), -1.5,\
                                1, 1, float(freq[i]))
            else:
                self.mk_file_af(self.tw_cell[cell_no], ('H', 'F', 'F', 'H'), -1.5,\
                                1, 0, float(freq[i]))
            self.run_autofish()
            disp_freq[i] = self.read_frequency()

        # ----- 分散関係にフィッティング -----
        popt, pcov = curve_fit(disp, disp_phase, disp_freq,\
                               (disp_freq[3],\
                                (disp_freq[6]-disp_freq[0])/disp_freq[3]))
        fa    = popt[0]
        kappa = popt[1]

        # ----- 加速モードの群速度の計算 -----
        vg =2.0*np.pi*fa*1e6/2.0*self.tw_cell[cell_no]['D']*1e-3*kappa*sin(2*np.pi/3)/\
             sqrt(1-kappa*cos(2*np.pi/3))

        # ----- 結果を保管 -----
        self.tw_cell[cell_no]['f0'] = disp_freq[0]
        self.tw_cell[cell_no]['f1'] = disp_freq[1]
        self.tw_cell[cell_no]['f2'] = disp_freq[2]
        self.tw_cell[cell_no]['f3'] = disp_freq[3]
        self.tw_cell[cell_no]['f4'] = disp_freq[4]
        self.tw_cell[cell_no]['f5'] = disp_freq[5]
        self.tw_cell[cell_no]['f6'] = disp_freq[6]
        self.tw_cell[cell_no]['fa'] = fa
        self.tw_cell[cell_no]['kappa'] = kappa
        self.tw_cell[cell_no]['disp_vg'] = vg/physics.c

        file_path = self.af_dir_30+'/'+self.sf_file+'{0:02d}'.format(cell_no)+'.af'
        os.replace(self.sf_file+'.af', file_path)

        print('\tfreq     0: {0:.6f}'.format(self.tw_cell[cell_no]['f0']))
        print('\t      pi/6: {0:.6f}'.format(self.tw_cell[cell_no]['f1']))
        print('\t     2pi/6: {0:.6f}'.format(self.tw_cell[cell_no]['f2']))
        print('\t     3pi/6: {0:.6f}'.format(self.tw_cell[cell_no]['f3']))
        print('\t     4pi/6: {0:.6f}'.format(self.tw_cell[cell_no]['f4']))
        print('\t     5pi/6: {0:.6f}'.format(self.tw_cell[cell_no]['f5']))
        print('\t        pi: {0:.6f}'.format(self.tw_cell[cell_no]['f6']))
        print('\tdisp    fa: {0:.6f}'.format(self.tw_cell[cell_no]['fa']))
        print('\t      kapa: {0:.6e}'.format(self.tw_cell[cell_no]['kappa']))
        print('\t      vg/c: {0:.6e}'.format(self.tw_cell[cell_no]['disp_vg']))

    # -------------------------------------------------------------------
    #  セル情報のリポートファイルを作成する．
    # -------------------------------------------------------------------
    def mk_report(self, path):
        ''' リポートファイルを作成する． '''
        name, ext = os.path.splitext(os.path.basename(path))
        file_path = self.report_dir+'/report_'+name+'.txt'
        report = open(file_path, 'w', encoding='utf-8')

        report.write('# {0:s}\n'.format(self.title))
        report.write('# {0:s}\n'.format(time.strftime('%Y/%m/%d %H:%M:%S')))
        report.write('# input file: {0:s}\n'.format(path))
        report.write('# mesh interval: {0:.3f} [mm]\n'.format(self.dx))
        report.write('\n\n\n')
        report.write('# '+90*'-'+'\n')
        report.write('# cell\tbeta\t2a\t2b\tD\tt\tfacc\tRs\tk\tvg\tQ\tr_ov_Q\n')
        report.write('# [ ]\t[vp/c]\t[mm]\t[mm]\t[mm]\t[mm]\t[MHz]\t[M Ohm/m]\t[V/(C cell)]\t[vg/c]\t[ ]\t[Ohm/cell]\n')
        report.write('# '+90*'='+'\n')
        for i in range(1, self.N_cell+1):
            report.write('{0:03d}\t'.format(i))
            report.write('{0:.3f}\t'.format(self.tw_cell[i]['beta']))
            report.write('{0:.4f}\t'.format(self.tw_cell[i]['2a']))
            report.write('{0:.6f}\t'.format(self.tw_cell[i]['2b']))
            report.write('{0:.4f}\t'.format(self.tw_cell[i]['D']))
            report.write('{0:.3f}\t'.format(self.tw_cell[i]['t']))
            report.write('{0:.6f}\t'.format(self.tw_cell[i]['facc']))
            report.write('{0:.3f}\t'.format(self.tw_cell[i]['Rs']))
            report.write('{0:.3e}\t'.format(self.tw_cell[i]['k']))
            report.write('{0:.6e}\t'.format(self.tw_cell[i]['vg']))
            report.write('{0:.3f}\t'.format(self.tw_cell[i]['Q']))
            report.write('{0:.3f}\n'.format(self.tw_cell[i]['r_ov_Q']))
        report.write('# '+90*'-'+'\n')

        if self.is_disp:
            report.write('\n\n\n')
            report.write('dispersion relation\n')
            report.write('# '+90*'-'+'\n')
            report.write('# cell\t2a\t0\tpi/6\t2pi/6\t3pi/6\t4pi/6\t5pi/6\tpi\tfa\tkappa\tvg/c\n')
            report.write('# [ ]\t[mm]\t[MHz]\t[MHz]\t[MHz]\t[MHz]\t[MHz]\t[MHz]\t[MHz]\t[MHz]\t[ ]\t[vg/c]\n')
            report.write('# '+90*'='+'\n')
            for i in range(1, self.N_cell+1):
                report.write('{0:03d}\t'.format(i))
                report.write('{0:.4f}\t'.format(self.tw_cell[i]['2a']))
                report.write('{0:.6f}\t'.format(self.tw_cell[i]['f0']))
                report.write('{0:.6f}\t'.format(self.tw_cell[i]['f1']))
                report.write('{0:.6f}\t'.format(self.tw_cell[i]['f2']))
                report.write('{0:.6f}\t'.format(self.tw_cell[i]['f3']))
                report.write('{0:.6f}\t'.format(self.tw_cell[i]['f4']))
                report.write('{0:.6f}\t'.format(self.tw_cell[i]['f5']))
                report.write('{0:.6f}\t'.format(self.tw_cell[i]['f6']))
                report.write('{0:.6f}\t'.format(self.tw_cell[i]['fa']))
                report.write('{0:.6e}\t'.format(self.tw_cell[i]['kappa']))
                report.write('{0:.6e}\n'.format(self.tw_cell[i]['disp_vg']))
            report.write('# '+90*'-'+'\n')

        if self.is_GPT:
            report.write('\n\n\n')
            report.write('for GPT calculation(TW)\n')
            report.write('# '+90*'-'+'\n')
            report.write('# No.\tzs\tze\tRF\tphase\talpha\tampSS\tampOO\tgdf_SS\tgdf_OO\tEz_S_SS\tEz_E_SS\tEz_S_OO\tEz_E_OO\n')
            report.write('#[ ]\t[mm]\t[mm]\t[W]\t[deg]\t[ ]\t[ ]\t[ ]\t[ ]\t[ ]\t[V/m]\t[V/m]\t[V/m]\t[V/m]\n')
            report.write('# '+90*'='+'\n')
            for i in range(1, self.N_cell+1):
                report.write('{0:02d}\t'.format(i))
                report.write('{0:.3f}\t'.format(self.tw_cell[i]['zs']))
                report.write('{0:.3f}\t'.format(self.tw_cell[i]['ze']))
                report.write('{0:.3e}\t'.format(self.tw_cell[i]['RF']))
                report.write('{0:.2f}\t'.format(self.tw_cell[i]['phase']))
                report.write('{0:.3e}\t'.format(self.tw_cell[i]['alpha']))
                report.write('{0:.3e}\t'.format(self.tw_cell[i]['ampSS']))
                report.write('{0:.3e}\t'.format(self.tw_cell[i]['ampOO']))
                report.write('{0:s}\t'.format(self.tw_cell[i]['gdf_SS']))
                report.write('{0:s}\t'.format(self.tw_cell[i]['gdf_OO']))
                report.write('{0:.3e}\t'.format(self.tw_cell[i]['Ez_S_SS']))
                report.write('{0:.3e}\t'.format(self.tw_cell[i]['Ez_E_SS']))
                report.write('{0:.3e}\t'.format(self.tw_cell[i]['Ez_S_OO']))
                report.write('{0:.3e}\n'.format(self.tw_cell[i]['Ez_E_OO']))
            report.write('# '+90*'-'+'\n')
                
            report.write('\n')
            report.write('for GPT calculation(SW)\n')
            report.write('# '+90*'-'+'\n')
            report.write('# cpl\tzs\tze\tphase\tamp\tgdf\n')
            report.write('# [ ]\t[mm]\t[mm]\t[deg]\t[ ]\t[ ]\n')
            report.write('# '+90*'='+'\n')
            for cpl in ('in', 'out'):
                report.write('{0:s}\t'.format(cpl))
                report.write('{0:.3f}\t'.format(self.sw_cell[cpl]['zs']))
                report.write('{0:.3f}\t'.format(self.sw_cell[cpl]['ze']))
                report.write('{0:.2f}\t'.format(self.sw_cell[cpl]['phase']))
                report.write('{0:.3e}\t'.format(self.sw_cell[cpl]['amp']))
                report.write('{0:s}\n'.format(self.sw_cell[cpl]['gdf']))
            report.write('# '+90*'-'+'\n')

        report.close()
        

    # -------------------------------------------------------------------
    # SUPERFISH の入力ファイル (*.af) の作成
    #    cell:   セルのデータ(辞書タイプ)
    #    cavity: ハーフセル(H), フルセル(F) を表すタプル. ('H', 'F', 'F', 'H')
    #    x0D:    左端のセル座標 z = x0*D
    #    nbslf:  左側境界条件 0:磁気的短絡 1:電気的短絡
    #    nbsrt:  右側境界条件
    #    freq:   float: ターゲット周波数  tuple: (start fr, stop fr, nstep)
    #    dx_fac: メッシュサイズ dx のファクター
    # -------------------------------------------------------------------
    def mk_file_af(self, cell, cavity, x0D, nbslf, nbsrt, freq, dx_fac=1.0):
        ''' SUPERFISH の入力ファイル (*.af) の作成する(3セル) '''
        
        # ----- メッシュサイズの変更 -----
        dx_old  = self.dx
        self.dx *= dx_fac

        # ----- 計算の準備 -----
        a, b, D, t = cell['2a']/2.0, cell['2b']/2.0, cell['D'], cell['t']
        x0 = x0D*D
        xreg, kreg, kmax, xmax = self.mk_xkreg(x0, cavity, a, b, D, t)
        yreg, lreg, lmax, ymax = self.mk_ylreg(a, b, t)
        xdriv, ydriv           = self.cal_xydriv(x0, xmax, cavity, b, D, nbslf, nbsrt)
        
        f = open(self.sf_file+'.af', 'w', encoding='utf-8')
        f.write('{0:s}\n'.format(self.title+': '+time.strftime('%Y/%m/%d %H:%M:%S') + '\n'))
        f.write('$reg kprob=1, ')
        f.write('dx={0:.6f}, '.format(self.dx))
        f.write('conv=0.1, ')
        f.write('kprob=1,\n')
        f.write(' xreg=')
        for xr in xreg: f.write('{0:.6f}, '.format(xr))
        f.write('\n')
        f.write(' kreg=')
        for kr in kreg: f.write('{0:d}, '.format(kr))
        f.write('\n')
        f.write(' kmax={0:d},\n'.format(kmax))
        f.write(' yreg={0:.6f}, {1:.6f}, \n'.format(yreg[0], yreg[1]))
        f.write(' lreg={0:d}, {1:d}, {2:d},\n'.format(lreg[0], lreg[1], lreg[2]))
        f.write(' lmax={0:d},\n'.format(lmax))
        f.write(' nbsup=1, nbslo=0, nbslf={0:d}, nbsrt={1:d},\n'.\
                format(nbslf, nbsrt))
        if type(freq) == float:
            f.write(' xdri={0:.6f}, ydri={1:.6f},\n'.format(xdriv, ydriv))
            f.write(' freq={0:.6f},\n'.format(freq))
        elif type(freq) == tuple:
            f.write(' nstep={0:d},\n'.format(freq[2]+1))
            f.write(' delfr={0:.6f},\n'.format((freq[1]-freq[0])/freq[2]))
            f.write(' freqs={0:.6f},\n'.format(freq[0]))
        else:
            print('\t\tfrequency type error at RF_Simulation.py > mk_file_af !!!')
            print('\t\t', type(freq), '\t', freq)
            exit()

        f.write(' kmethod=1,\n')
        f.write(' beta={0:.6f}$\n\n'.format(cell['beta']))

        self.mk_cell_point(f, x0, cavity, a, b, D, t)

        self.dx = dx_old      # メッシュサイズを元に
        f.close()



    # -------------------------------------------------------------------
    # xreg, kreg, kmax の計算
    # -------------------------------------------------------------------
    def mk_xkreg(self, x0, cavity, a, b, D, t):
        ''' xreg, kreg, kmax の計算を行う '''
        def error(prt_str):
            print('\n\nerror occured at RF_Simulation.py > mk_cell_point')
            print(prt_str)
            exit()

        xe = x0
        nc = 0
        xreg = []

        for ct in cavity:
            if ct=='H' and nc==0:
                xreg.append(xe+0.5*D-0.5*t)
                xe += 0.5*D
                nc += 1
            elif ct=='F':
                xreg.append(xe)
                xreg.append(xe+0.5*t)
                xreg.append(xe+0.5*D)
                xreg.append(xe+D-0.5*t)
                xe += D
                nc += 1
            elif ct == 'H' and nc == len(cavity)-1:
                xreg.append(xe)
                xreg.append(xe+0.5*t)
                xe += 0.5*D
                nc += 1

        xmax = xe
        x = copy.deepcopy(xreg)
        x[:0] = [x0]
        x.append(xmax)
        nreg = [int((x[i+1]-x[i])/self.dx) for i in range(0, len(x)-1)]
        Nreg = []
        sum_neg = 0
        for n in nreg:
            sum_neg += n
            Nreg.append(sum_neg)
        
        kreg = [Nreg[i] for i in range(0, len(Nreg)-1)]
        kreg[:0] = [0]
        kmax = Nreg[-1]

        return xreg, kreg, kmax, xmax

    # -------------------------------------------------------------------
    # yreg, lreg, lmax の計算
    # -------------------------------------------------------------------
    def mk_ylreg(self, a, b, t):
        ''' yreg, lreg, lmax の計算を行う '''

        yreg = [a, a+0.5*t]
        lreg = [int(yreg[i]/self.dx) for i in range(0, len(yreg))]
        lreg[:0] = [0]
        lmax = int(b/self.dx)
        ymax = b

        return yreg, lreg, lmax, ymax


    # -------------------------------------------------------------------
    # ドライビングポイントの計算
    # -------------------------------------------------------------------
    def cal_xydriv(self, x0, xmax, cavity, b, D, nbslf, nbsrt):
        ''' ドライビングポイントを計算します '''

        if cavity[0]    == 'H' and nbslf == 1: xdriv = x0
        elif cavity[-1] == 'H' and nbsrt == 1: xdriv = xmax
        elif cavity[0]  == 'F'               : xdriv = x0+0.5*D
        elif cavity[-1] == 'F'               : xdriv = xmax-0.5*D
        else                                 : xdriv = x0+D

        if xdriv < x0 or xmax < xdriv:
            print('\n\n Error RF_Simulation.py > cal_xydrive !!!')
            print('x0: {0:.6f}\txmax: {1:.6f}\txdriv: {2:.6f}'.\
                  format(x0, xmax, xdriv))
            
        return xdriv, 0.99*b
    
    # -------------------------------------------------------------------
    # セルのポイント座標の作成
    # -------------------------------------------------------------------
    def mk_cell_point(self, f, x0, cavity, a, b, D, t):
        ''' セルのポイント (一筆書き) を作成する '''
        def error(prt_str):
            print('\n\nerror occured at RF_Simulation.py > mk_cell_point')
            print(prt_str)
            exit()

        xe = x0
        if cavity[0]=='H':
            f.write('$po x={0:.6f}, y={1:.6f} $\n'.format(xe, 0.0))
        elif cavity[0]=='F':
            f.write('$po x={0:.6f}, y={1:.6f} $\n'.format(xe, 0.0))
            f.write('$po x={0:.6f}, y={1:.6f} $\n'.format(xe, a))
        else:
            error('\n\nerror cell type')
        
        nc = 0
        for ct in cavity:
            if ct=='H' and nc==0:
                f.write('!---------- half cell -----------\n')
                f.write('$po x={0:.6f}, y={1:.6f} $\n'.format(xe, b))
                f.write('$po x={0:.6f}, y={1:.6f} $\n'.format(xe+0.5*D-0.5*t, b))
                f.write('$po x={0:.6f}, y={1:.6f} $\n'.format(xe+0.5*D-0.5*t, a+0.5*t))
                f.write('$po nt=2, r={0:.6f}, theta=270, x0={1:.6f}, y0={2:.6f} $\n'\
	                .format(0.5*t, xe+0.5*D, a+0.5*t))
                xe += 0.5*D
                nc += 1
            elif ct == 'F':
                f.write('!---------- full cell -----------\n')
                f.write('$po nt=2, r={0:.6f}, theta=0, x0={1:.6f}, y0={2:.6f} $\n'\
	                .format(0.5*t, xe, a+0.5*t))
                f.write('$po x={0:.6f}, y={1:.6f} $\n'.format(xe+0.5*t, b))
                f.write('$po x={0:.6f}, y={1:.6f} $\n'.format(xe+D-0.5*t, b))
                f.write('$po x={0:.6f}, y={1:.6f} $\n'.format(xe+D-0.5*t, a+0.5*t))
                f.write('$po nt=2, r={0:.6f}, theta=270, x0={1:.6f}, y0={2:.6f} $\n'\
	                .format(0.5*t, xe+D, a+0.5*t))
                xe += D
                nc += 1
            elif ct == 'H' and nc == len(cavity)-1:
                f.write('!---------- half cell -----------\n')
                f.write('$po nt=2, r={0:.6f}, theta=0, x0={1:.6f}, y0={2:.6f} $\n'\
	                .format(0.5*t, xe, a+0.5*t))
                f.write('$po x={0:.6f}, y={1:.6f} $\n'.format(xe+0.5*t, b))
                f.write('$po x={0:.6f}, y={1:.6f} $\n'.format(xe+0.5*D, b))
                xe += 0.5*D
                nc += 1
            else:
                error('\n\nerror cell type') 

        f.write('!--------------------------------\n')
        f.write('$po x={0:.6f}, y={1:.6f} $\n'.format(xe, 0.0))
        f.write('$po x={0:.6f}, y={1:.6f} $\n'.format(x0, 0.0))

        
    # -------------------------------------------------------------------
    # autofish の実行
    # -------------------------------------------------------------------
    def run_autofish(self):
        ''' autofish を実行する '''
        command = ["autofish"]
        command.append(self.sf_file+'.af')

        # --- autofish の実行 -----
        res_af = subprocess.run(command, universal_newlines=True, \
                                 stdout=subprocess.PIPE)
        if res_af.returncode != 0:
            print("autofish error !!!")
            exit(99)

    # -------------------------------------------------------------------
    # sf7 の実行
    # -------------------------------------------------------------------
    def run_sf7(self):
        ''' sf7 を実行する '''
        command = ["sf7"]

        # --- sf7 の実行 -----
        res_af = subprocess.run(command, universal_newlines=True, \
                                 stdout=subprocess.PIPE)
        if res_af.returncode != 0:
            print("fish2gdf error !!!")
            exit(99)

    # -------------------------------------------------------------------
    # fish2gdf の実行
    # -------------------------------------------------------------------
    def run_fish2gdf(self, gdf_file, sf7_file):
        ''' autofish を実行する '''
        command = ['fish2gdf']
        command.append('-o')
        command.append(gdf_file)
        command.append(sf7_file)

        # --- autofish の実行 -----
        res_af = subprocess.run(command, universal_newlines=True, \
                                 stdout=subprocess.PIPE)
        if res_af.returncode != 0:
            print("autofish error !!!")
            exit(99)
            
    # -------------------------------------------------------------------
    # READ rough resonance frequnecy (form OUTFIS.TXT)
    # -------------------------------------------------------------------
    def read_resonance_frequency_form_fcan(self):
        ''' OUTFIS.TXT から読み込んだラフな共振周波数を返す． '''
        target = re.compile(r'^Frequency\s+k\^2\s+D\(k\^2\)')
        rd_file = 'OUTFIS.TXT'
        fd = open(rd_file, 'r')
        ln = 0
        for line in fd:
            line = line.strip()
            ln += 1
            if re.search(target, line):
                fd.close()
                break
            
        dat = np.genfromtxt(rd_file, skip_header=ln)
        resonance = []
        for i in range(1, len(dat)):
            if  dat[i][2] <= 0.0 <= dat[i-1][2]:
                resonance.append(dat[i-1][0]-
                                 (dat[i-1][0]-dat[i][0])/(dat[i-1][2]-dat[i][2])*dat[i-1][2])

        return resonance

    
    # -------------------------------------------------------------------
    # READ frequency (***.SFO)
    # -------------------------------------------------------------------
    def read_frequency(self):
        ''' *.SFO から周波数を読み込み返す． '''
        target = re.compile(r'^Frequency\s+=')
        SFO = open(self.sf_file+'.SFO', 'r')
        for line in SFO:
            line = line.strip()
            if re.search(target, line): break

        line = re.split('\s+', line)
        frequency = float(line[2])
        SFO.close()
        return frequency

        
    # -------------------------------------------------------------------
    # READ SFO
    #    f [MHz]: resonant frequency
    #    U [Joules]: Stored Energy
    #    Q: Quality value
    #    ZTT [MOhm/m]: Shunt Impedance
    #    r_ov_Q [Ohm]
    # -------------------------------------------------------------------
    def read_SFO(self):
        ''' *.SFO を読んで [f, U, ,Q, ZTT, r_ov_Q] を返す． '''
        target = re.compile(r'^All calculated values below refer to the mesh geometry only.')
        SFO = open(self.sf_file+'.SFO', 'r')

        for rline in SFO:
            rline = rline.strip()
            if re.search(target, rline): break

        for rline in SFO:
            rline = rline.strip()
            if re.search('^Frequency\s+=', rline):
                rline = re.split('\s+', rline)
                f = float(rline[2])
            elif re.search('^Stored\s+energy\s+=', rline):
                rline = re.split('\s+', rline)
                U = float(rline[3])
            elif re.search('^Q\s+=', rline):
                rline = re.split('\s+',rline)
                Q = float(rline[2])
            elif re.search('Z\*T\*T', rline):
                rline = re.split('\s+',rline)
                ZTT = float(rline[6])
            elif re.search('r\/Q', rline):
                rline = re.split('\s+',rline)
                r_ov_Q = float(rline[2])
            elif re.search('^Wall segments:', rline):break
            
        SFO.close()
        
        return f, U, Q, ZTT, r_ov_Q 

    # -------------------------------------------------------------------
    # make seg file for calculating Q factor
    # -------------------------------------------------------------------
    def mk_seg(self, seg_start, seg_end):
        ''' Q値計算用の seg ファイルの作成 '''
        SEG = open(self.sf_file+'.seg', 'w')

        SEG.write('FieldSegments\n')
        for i in range(seg_start, seg_end+1):
            SEG.write('{0:d} '.format(i))
        SEG.write('\nEndData\n')
        SEG.write('End\n')

        SEG.close()

    # -------------------------------------------------------------------
    # make seg file for calculating vg
    # -------------------------------------------------------------------
    def mk_seg2(self, segL, segR):
        ''' 群速度計算用の seg ファイルの作成 '''
        SEG = open(self.sf_file+'.seg', 'w')

        SEG.write('FieldSegments\n')
        SEG.write('{0:d} {1:d}\n'.format(segL, segR))
        SEG.write('\nEndData\n')
        SEG.write('End\n')

        SEG.close()


    # -------------------------------------------------------------------
    #    read Ez at side
    # -------------------------------------------------------------------
    def side_Ez(self, min_z, max_z):
        ''' 端の Ez を読み込む． '''
        r  = 0.0;
        nz = 1

        IN7 = open(self.sf_file+'.in7', 'w')
        IN7.write('line noscreen\n')
        IN7.write('{0:.3f}\t{1:.3f}\t{2:.3f}\t{3:.3f}\n'.format(min_z, r ,max_z, r))
        IN7.write('{0:d}\n'.format(nz))
        IN7.write('end')
        IN7.close()

        command = ["sf7"]
        res = subprocess.run(command, universal_newlines=True, \
                                 stdout=subprocess.PIPE)
        if res.returncode != 0:
            print("sf7 error !!!")
            exit(99)

        SFOUT = open('OUTSF7.TXT', 'r')

        for line in SFOUT:
            line = line.strip()
            if re.search(r'^\(mm\)\s+\(mm\)', line): break

        line = SFOUT.readline()
        line = line.strip()
        line = re.split('\s+', line)
        Ez_S = float(line[2])

        line = SFOUT.readline()
        line = line.strip()
        line = re.split('\s+', line)
        Ez_E = float(line[2])

        SFOUT.close()

        return Ez_S, Ez_E


    # -------------------------------------------------------------------
    #    set Er or Ht field
    #      fld:   Er or Ht
    #      z:     Ht/Er を格納する z座標
    #      rmin:                  r座標(最小値)
    #      rmax:                  r座標(最大値)
    # -------------------------------------------------------------------
    def set_field(self, fld, z, rmin, rmax):
        ''' 端の電磁場を読み込む． '''
        if   fld == 'Er':
            idx, fac = 3, 1e+6
        elif fld == 'Ht':
            idx, fac = 5, 1.0
        else:
            print('Error occured at set_field !!\n')
            print('1st arg of set_field should be \'Er\' or \'Ht\'\n')
            print('\t arg:{0:s}\n'.format(fld))
            exit()

        dr   = (rmax - rmin)/self.N_divR

        IN7 = open(self.sf_file+'.in7', 'w')
        IN7.write('line noscreen\n')
        IN7.write('{0:.6f}\t{1:.6f}\t{2:.6f}\t{3:.6f}\n'.format(z, rmin ,z, rmax))
        IN7.write('{0:d}\n'.format(self.N_divR))
        IN7.write('end')
        IN7.close()

        command = ["sf7"]
        res = subprocess.run(command, universal_newlines=True, \
                                 stdout=subprocess.PIPE)
        if res.returncode != 0:
            print("sf7 error !!!")
            exit(99)

        SFOUT = open('OUTSF7.TXT', 'r')
        for line in SFOUT:
            line = line.strip()
            if re.search(r'^\(mm\)\s+\(mm\)\s+\(MV\/m\)\s+\(MV\/m\)\s+\(MV\/m\)\s+\(A\/m\)',\
                         line): break

        r = []
        EH = []
        for i in range(0, self.N_divR+1):
            line = SFOUT.readline()
            line = line.strip()
            if re.search('^\s*\n',line): break
            line = re.split('\s+', line)
            line = [float(line[i]) for i in range(0, len(line))]
            if 0.001 < np.abs(line[0]-z) or 0.001 < abs(line[1]-i*dr):
                print('Error occured at set_field !!\n')
                print('\t\t{0:d}\t\tZ = {1:.6f}\t\t R = {2:.6f}\n'.\
                      format(i, line[0], line[1]))
                exit()
            r.append(line[1])
            EH.append(fac*line[idx])
            
        SFOUT.close()
        if 0.001<np.abs(r[0]-rmin) or 0.001<np.abs(r[self.N_divR]-rmax):
            print('Error occured at set_field !!\n')
            print('r[0]:{0:.6f}\t\trmin:{1:.6f}\n'.format(r[0], rmin))
            print('r[{0:d}]:{1:.6f}\t\trmax:{2:.6f}\n'.\
                  format(self.N_divR, r[self.N_divR], rmax))

        return r, EH


    # -------------------------------------------------------------------
    #    calculate RF power case of stored energy 2 [Joules] in 1.5 cells
    #    U_SS:  short-short の時の蓄積エネルギー [J]
    #    U_OO:  open-open の時の蓄積エネルギー [J]
    #
    # -------------------------------------------------------------------
    def cal_RF_pwr(self,
                   U_SS, rL_SS, rR_SS, HL_SS, HR_SS,\
                   U_OO, rL_OO, rR_OO, EL_OO, ER_OO):
        ''' RF電力 (ポインティングベクトル) の計算を行う. '''
        U_unit = 1                         # [Joules]
        fac_H = sqrt(U_unit/U_SS)
        fac_E = sqrt(U_unit/U_OO)

        # ----- left side calculation
        pwrL = 0.0;
        for i in range(0, self.N_divR):
            E = fac_E*(EL_OO[i]+EL_OO[i+1])/2.0
            H = fac_H*(HL_SS[i]+HL_SS[i+1])/2.0
            if 0.001<np.abs(rL_SS[i]-rL_OO[i]):
                print('error occured at cal_RF_pwr !!\n')
                print('\t left side r coodinate.\n')
                print('\t{0:d}\trL_SS:{1:.6f}\trL_OO:{2:.6f}\n'.\
                      format(i, rL_SS[i], rL_OO[i]))
                exit(0)
            r  = (rL_SS[i]+rL_SS[i+1])/2.0*1e-3   # r  [m]
            dr = (rL_SS[i+1]-rL_SS[i])*1e-3       # dr [m]
            pwrL += 2*np.pi*r*dr*E*H
        pwrL /= 2.0

        # -----  Right side calculation
        pwrR = 0.0
        for i in range(0, self.N_divR):
            E = fac_E*(ER_OO[i]+ER_OO[i+1])/2.0
            H = fac_H*(HR_SS[i]+HR_SS[i+1])/2.0
            if 0.001<np.abs(rR_SS[i]-rR_OO[i]):
                print('error occured at cal_RF_pwr !!\n')
                print('\t right side r coodinate.\n')
                print('\t{0:d}\trR_SS:{1:.6f}\trR_OO:{2:.6f}\n'.\
                      format(rR_SS[i], rR_OO[i]))
                exit()
            r  = (rR_SS[i]+rR_SS[i+1])/2.0*1e-3   # r  [m]
            dr = (rR_SS[i+1]-rR_SS[i])*1e-3       # dr [m]
            pwrR += 2.0*np.pi*r*dr*E*H
        pwrR /= 2.0

        return U_unit*2.0, pwrL, pwrR

    # -------------------------------------------------------------------
    # gdf ファイルの作成
    # -------------------------------------------------------------------
    def mk_gdf(self, gdf_file, max_z, max_r):
        ''' gdf ファイルを作成する '''

        nz = int(max_z/self.GPT['mesh_size'])
        nr = int(max_r/self.GPT['mesh_size'])

        IN7 = open(self.sf_file+'.in7', 'w', encoding='utf-8')
        IN7.write('rect noscreen\n')
        IN7.write('{0:.3f}\t{1:.3f}\t{2:.3f}\t{3:.3f}\n'.format(0.0, 0.0 ,max_z, max_r))
        IN7.write('{0:d}\t{1:d}\n'.format(nz, nr))
        IN7.write('end')
        IN7.close()

        self.run_sf7()
        self.run_fish2gdf(os.path.join(self.GPT_dir, gdf_file), 'OUTSF7.TXT')


    # -------------------------------------------------------------------
    #  カップラーの半分の定在波領域の初期化
    # -------------------------------------------------------------------
    def init_SW_region(self):
        ''' カップラーの半分の定在波領域の初期化をおこなう '''
        self.sw_cell={}
        self.sw_cell['in'] = copy.deepcopy(SUPERFISH.cpl)
        self.sw_cell['in']['p_len'] = self.GPT['in_cpl']['p_len']
        self.sw_cell['in']['p_dia'] = self.GPT['in_cpl']['p_dia']
        self.sw_cell['in']['ra'] = self.tw_cell[1]['t']/2.0
        self.sw_cell['in']['D_half'] = (self.tw_cell[1]['D'] - self.tw_cell[1]['t'])/2.0
        self.sw_cell['in']['2b'] = self.tw_cell[1]['2b']
        self.sw_cell['in']['2a'] = self.tw_cell[1]['2a']

        self.sw_cell['out'] = copy.deepcopy(SUPERFISH.cpl)
        self.sw_cell['out']['p_len'] = self.GPT['out_cpl']['p_len']
        self.sw_cell['out']['p_dia'] = self.GPT['out_cpl']['p_dia']
        self.sw_cell['out']['ra'] = self.tw_cell[self.N_cell]['t']/2.0
        self.sw_cell['out']['D_half'] = (self.tw_cell[self.N_cell]['D']\
                                           - self.tw_cell[self.N_cell]['t'])/2.0
        self.sw_cell['out']['2b'] = self.tw_cell[self.N_cell]['2b']
        self.sw_cell['out']['2a'] = self.tw_cell[self.N_cell]['2a']

        
    # -------------------------------------------------------------------
    #  カップラーの半分の定在波領域の計算
    # -------------------------------------------------------------------
    def mk_sw_region(self):
        ''' カップラーの半分の定在波領域の計算を行う '''
        
        print('\n\n----- RF input coupler SW region ------------\n')
        self.mk_seg(2, 5)
        self.mk_file_af_inSW()
        self.run_autofish()
        freq, U, Q, ZTT, r_ov_Q = self.read_SFO()
        Ez_S, Ez_E = self.side_Ez(0, self.sw_cell['in']['p_len'] + \
                                  self.sw_cell['in']['D_half'])

        print('\t Frequency    \t{0:.5f}'.format(freq))
        print('\t Stored Energy\t{0:e}'.format(U))
        print('\t Q value      \t{0:.2f}'.format(Q))
        print('\t ZTT          \t{0:e}'.format(ZTT))
        print('\t r/Q          \t{0:e}'.format(r_ov_Q))
        print('\t Ez(start)    \t{0:e}'.format(Ez_S))
        print('\t Ez(end)      \t{0:e}'.format(Ez_E))

        self.sw_cell['in']['freq']   = freq
        self.sw_cell['in']['U']      = U
        self.sw_cell['in']['Q']      = Q
        self.sw_cell['in']['Rs']     = 2*ZTT
        self.sw_cell['in']['r_ov_Q'] = r_ov_Q
        self.sw_cell['in']['Ez_S']   = Ez_S
        self.sw_cell['in']['Ez_E']   = Ez_E
        self.sw_cell['in']['gdf']    = 'inSW.gdf'
        self.mk_gdf(self.sw_cell['in']['gdf'],
                    self.sw_cell['in']['p_len'] + self.sw_cell['in']['D_half'],
                    self.sw_cell['in']['2a']/2.0+1)
        os.replace(self.sf_file+'.af', self.af_dir_sw+'/'+'inSW.af')
        
        print('\n\n----- RF output coupler SW region ------------\n')
        self.mk_seg(4, 7)
        self.mk_file_af_outSW()
        self.run_autofish()
        freq, U, Q, ZTT, r_ov_Q = self.read_SFO()
        Ez_S, Ez_E = self.side_Ez(0, self.sw_cell['in']['p_len'] + \
                                  self.sw_cell['in']['D_half'])
        print('\t Frequency    \t{0:.5f}'.format(freq))
        print('\t Stored Energy\t{0:e}'.format(U))
        print('\t Q value      \t{0:.2f}'.format(Q))
        print('\t r/Q          \t{0:e}'.format(r_ov_Q))
        print('\t Rs           \t{0:e}'.format(ZTT*2))
        print('\t Ez(start)    \t{0:e}'.format(Ez_S))
        print('\t Ez(end)      \t{0:e}'.format(Ez_E))
        
        self.sw_cell['out']['freq']   = freq
        self.sw_cell['out']['U']      = U
        self.sw_cell['out']['Q']      = Q
        self.sw_cell['out']['Rs']     = 2*ZTT
        self.sw_cell['out']['r_ov_Q'] = r_ov_Q
        self.sw_cell['out']['Ez_S']   = Ez_S
        self.sw_cell['out']['Ez_E']   = Ez_E
        self.sw_cell['out']['gdf']    = 'outSW.gdf'
        self.mk_gdf(self.sw_cell['out']['gdf'],
                    self.sw_cell['out']['p_len'] + self.sw_cell['out']['D_half'],
                    self.sw_cell['out']['2a']/2.0+1)
        os.replace(self.sf_file+'.af', self.af_dir_sw+'/'+'outSW.af')
        


    # -------------------------------------------------------------------
    #  入力カップラーの上流半分 (SW) を計算する
    # -------------------------------------------------------------------
    def mk_file_af_inSW(self):
        ''' SUPERFISH の入力ファイル (*.af) の作成する (パイプ+ハーフセル)． '''

        pl = self.sw_cell['in']['p_len']
        a  = self.sw_cell['in']['p_dia']/2.0
        ra = self.sw_cell['in']['ra']
        b  = self.sw_cell['in']['2b']/2.0
        D  = self.sw_cell['in']['D_half']
        dx = self.dx

        nbslf = 1
        nbsrt = 1
        xdri  = 0.99*(pl+D)
        ydri  = 0.99*b

        xreg  = [pl-ra, pl]
        kreg  = [0, int(xreg[0]/dx), int(xreg[1]/dx)]
        kmax = int((pl+D)/dx)

        yreg  = [a, a+ra]
        lreg  = [0, int(a/dx), int((a+ra)/dx)]
        lmax  = int(b/dx)

        f = open(self.sf_file+'.af', 'w', encoding='utf-8')

        f.write('{0:s} (input SW region)\n'.format(self.title));
        f.write('$reg kprob=1, ')
        f.write(' dx={0:.6f},'.format(self.dx))
        f.write(' conv=0.1, ') 
        f.write(' kprob=1,\n')
        f.write(' xreg={0:.6f}, {1:.6f},\n'.format(xreg[0], xreg[1]))
        f.write(' kreg={0:d}, {1:d}, {2:d},\n'.format(kreg[0], kreg[1], kreg[2]))
        f.write(' kmax={0:d},\n'.format(kmax))
        f.write(' yreg={0:.6f}, {1:.6f},\n'.format(yreg[0], yreg[1]))
        f.write(' lreg={0:d}, {1:d}, {2:d},\n'.format(lreg[0], lreg[1], lreg[2]))
        f.write(' lmax={0:d},\n'.format(lmax))

        f.write(' xdri={0:.6f}, ydri={1:.6f},\n'.format(xdri, ydri))
        f.write(' nbsup=1, nbslo=0, nbslf={0:d}, nbsrt={0:d}\n'.format(nbslf, nbsrt))
        f.write(' freq={0:.6f},\n'.format(self.facc))
        f.write(' kmethod=1,\n')
        f.write(' beta={0:.6f}$\n\n'.format(self.tw_cell[1]['beta']))


        f.write('!---------- input SW region -----------\n')
        f.write(' $po x={0:.6f}, y={1:.6f} $\n'.format(0.0, 0.0))
        f.write(' $po x={0:.6f}, y={1:.6f} $\n'.format(0.0, a))
        f.write(' $po x={0:.6f}, y={1:.6f} $\n'.format(pl-ra, a))
        f.write(' $po nt=2, r={0:.6f}, theta=0, x0={1:.6f}, y0={2:.6f} $\n'.\
                format(ra, pl-ra, a+ra))
        f.write(' $po x={0:.6f}, y={1:.6f} $\n'.format(pl, b))
        f.write(' $po x={0:.6f}, y={1:.6f} $\n'.format(pl+D, b))
        f.write(' $po x={0:.6f}, y={1:.6f} $\n'.format(pl+D, 0.0))
        f.write(' $po x={0:.6f}, y={1:.6f} $\n'.format(0.0, 0.0))

        f.close()


    # -------------------------------------------------------------------
    # 出力カップラーの上流半分 (SW) を計算する
    # -------------------------------------------------------------------
    def mk_file_af_outSW(self):
        ''' SUPERFISH の出力ファイル (*.af) の作成する (パイプ+ハーフセル)． '''

        pl = self.sw_cell['out']['p_len']
        a  = self.sw_cell['out']['p_dia']/2.0
        ra = self.sw_cell['out']['ra']
        b  = self.sw_cell['out']['2b']/2.0
        D  = self.sw_cell['out']['D_half']
        dx = self.dx

        nbslf = 1
        nbsrt = 1
        xdri  = 0.0
        ydri  = 0.99*b

        xreg  = [D, D+ra]
        kreg  = [0, int(xreg[0]/dx), int(xreg[1]/dx)]
        kmax = int((pl+D)/dx)

        yreg  = [a, a+ra]
        lreg  = [0, int(a/dx), int((a+ra)/dx)]
        lmax  = int(b/dx)

        f = open(self.sf_file+'.af', 'w', encoding='utf-8')

        f.write('{0:s} (output SW region)\n'.format(self.title));
        f.write('$reg kprob=1, ')
        f.write(' dx={0:.6f},'.format(self.dx))
        f.write(' conv=0.1, ') 
        f.write(' kprob=1,\n')
        f.write(' xreg={0:.6f}, {1:.6f},\n'.format(xreg[0], xreg[1]))
        f.write(' kreg={0:d}, {1:d}, {2:d},\n'.format(kreg[0], kreg[1], kreg[2]))
        f.write(' kmax={0:d},\n'.format(kmax))
        f.write(' yreg={0:.6f}, {1:.6f},\n'.format(yreg[0], yreg[1]))
        f.write(' lreg={0:d}, {1:d}, {2:d},\n'.format(lreg[0], lreg[1], lreg[2]))
        f.write(' lmax={0:d},\n'.format(lmax))

        f.write(' xdri={0:.6f}, ydri={1:.6f},\n'.format(xdri, ydri))
        f.write(' nbsup=1, nbslo=0, nbslf={0:d}, nbsrt={0:d}\n'.format(nbslf, nbsrt))
        f.write(' freq={0:.6f},\n'.format(self.facc))
        f.write(' kmethod=1,\n')
        f.write(' beta={0:.6f}$\n\n'.format(self.tw_cell[self.N_cell]['beta']))

        f.write('!---------- output SW region -----------\n')
        f.write(' $po x={0:.6f}, y={1:.6f} $\n'.format(0.0, 0.0))
        f.write(' $po x={0:.6f}, y={1:.6f} $\n'.format(0.0, b))
        f.write(' $po x={0:.6f}, y={1:.6f} $\n'.format(D, b))
        f.write(' $po x={0:.6f}, y={1:.6f} $\n'.format(D, a+ra))
        f.write(' $po nt=2, r={0:.6f}, theta=270, x0={1:.6f}, y0={2:.6f} $\n'\
                .format(ra, D+ra, a+ra))
        f.write(' $po x={0:.6f}, y={1:.6f} $\n'.format(D+pl, a))
        f.write(' $po x={0:.6f}, y={1:.6f} $\n'.format(pl+D, 0.0))
        f.write(' $po x={0:.6f}, y={1:.6f} $\n'.format(0.0, 0.0))

        f.close()

    # -------------------------------------------------------------------
    # GPT のインプットファイル (*.in) を作るための初期設定
    # -------------------------------------------------------------------
    def GPT_in_init(self):
        ''' GPT のインプットファイル (*.in) を作るための初期設定をする． '''

        self.gpt_in_path = os.path.join(self.GPT_dir, 'gpt_in.txt')
        self.init_phase = self.GPT['InitP'];     # No.1 cell phase  [deg]
        self.phase_shift = -120.0;               # phase shift/cell [deg]
        self.dp_ri = -90;                        # phase difference between real and imag

        f = open(self.gpt_in_path, 'w')
        f.write('#=========================================================\n')
        f.write('# TW Acelerating Structure with beam loading\n')
        f.write('#      Input RF: {0:.6f} [MW]\n'.format(self.GPT['RF_PWR']*1e-6))
        f.write('#      Beam current: {0:.6f} [A]\n'.format(self.GPT['Ib']))
        f.write('#==========================================================\n')
        f.write('degree = pi/180;\n')
        f.write('mm = 1.0e-3;\n')
        f.write('offset_z = 0.0;  # set offset z\n')
        f.write('offset_p = 0.0;  # set offset phase\n\n')
        f.close()


    # -------------------------------------------------------------------
    # GPTインプット用の進行波の部分を計算する
    # -------------------------------------------------------------------
    def set_cell_para(self):
        ''' GPTインプット用の進行波の部分を計算する '''
        
        # ----- セルごとの蓄積エネルギーと RF 電力の計算 -----
        self.cal_tw_cell_RF_power_beam_loading()

        # ----- セル毎の (座標，位相，減衰定数，SF比の電磁場強度) の計算 -----
        ze  = self.GPT['StrZ'] + self.tw_cell[1]['D']/2.0 - self.tw_cell[1]['t']/2.0
        phase = self.init_phase - self.phase_shift
        for i in range(1, self.N_cell+1):
	    #-------- set phase ---------
            phase += self.phase_shift
            self.tw_cell[i]['phase'] = phase

            #-------- set z coordinate ---------
            zs, L = ze, self.tw_cell[i]['D']
            ze = zs + L
            zc = 0.5*(zs + ze)
            self.tw_cell[i]['zs'] = zs
            self.tw_cell[i]['ze'] = ze

            #-------- set attenuation parameters ---------
            omega  = 2*np.pi*self.facc*1e6
            light  = physics.c
            vg    = self.tw_cell[i]['vg']
            Q     = self.tw_cell[i]['Q']
            alpha = omega/(2.0 * vg*light * Q * self.GPT['Qfac'])
            self.tw_cell[i]['alpha'] = alpha

	
	    #-------- set field multiplication factor  ---------
            U_SS = self.tw_cell[i]['U_SS']           # 1.5 cell Stored energy
            U_OO = self.tw_cell[i]['U_OO']           # 1.5 cell Stored energy
            U = self.tw_cell[i]['U']*1.5             # 1.5 cell Stored energy
            sign_SS = -1 if self.tw_cell[i]['Ez_E_SS'] <= 0 else 1
            sign_OO = -1 if self.tw_cell[i]['Ez_E_OO'] <= 0 else 1
            self.tw_cell[i]['ampSS'] =  sign_SS*sqrt(0.5*U/U_SS)
            self.tw_cell[i]['ampOO'] = -sign_OO*sqrt(0.5*U/U_OO)


    # -------------------------------------------------------------------
    # ビームローディングを考慮した蓄積エネルギー
    # -------------------------------------------------------------------
    def cal_tw_cell_RF_power_beam_loading(self):
        ''' 進行波のセルの蓄積エネルギーを計算する '''

        w = 2*np.pi*self.facc*1e6
        c = physics.c

        # ----- ビームローディングが無いときの蓄積エネルギーの計算 -----
        m = np.zeros((self.N_cell, self.N_cell))
        l = np.zeros(self.N_cell)
        Pin = self.GPT['RF_PWR']
        for i in range(1, self.N_cell+1):
            vg = self.tw_cell[i]['vg'] * c
            Q  = self.tw_cell[i]['Q'] * self.GPT['Qfac']
            D  = self.tw_cell[i]['D'] * 1e-3
            if i == 1:
                m[i-1][i-1] = -(w/Q + vg/D)
                l[i-1] = -Pin
            else:
                vgm = self.tw_cell[i-1]['vg'] * c
                Dm  = self.tw_cell[i-1]['D']  * 1e-3
                m[i-1][i-2] = vgm/Dm
                m[i-1][i-1] = -(w/Q + vg/D)
        Usol = solve(m, l)

        # ----- ビームローディングが無いときの蓄積エネルギーの格納 -----
        for i in range(1, self.N_cell+1):
            self.tw_cell[i]['U'] = Usol[i-1]

        # ----- ビームローディングが有るときの蓄積エネルギーの計算 -----
        change_sum_U = 1.0
        DT = 1/(self.facc*1e6)
        T  = 0.0
        Ib  = self.GPT['Ib']           # [A]
         
        while 1e-6 < change_sum_U:
            T += DT
            change_sum_U = 0.0

            vg  = self.tw_cell[1]['vg'] * c
            Q   = self.tw_cell[1]['Q'] * self.GPT['Qfac']
            D   = self.tw_cell[1]['D'] * 1e-3
            k   = self.tw_cell[1]['k']
            U   = self.tw_cell[1]['U']
            V   = np.sqrt(4*U*k)
            
            self.tw_cell[1]['U'] = U + (Pin - w*U/Q -vg/D*U -Ib*V)*DT - k*(Ib*DT*DT) 
            change_sum_U += np.abs((self.tw_cell[1]['U']-U)/U)
            
            for i in range(2, self.N_cell+1):
                vg  = self.tw_cell[i]['vg']*c
                vgm = self.tw_cell[i-1]['vg']*c
                Q   = self.tw_cell[i]['Q']*self.GPT['Qfac']
                D   = self.tw_cell[i]['D']*1e-3
                Dm  = self.tw_cell[i-1]['D']*1e-3
                k   = self.tw_cell[i]['k']
                U   = self.tw_cell[i]['U']
                Um  = self.tw_cell[i-1]['U']
                V   = np.sqrt(4*U*k)
                self.tw_cell[i]['U'] = U + (vgm/Dm*Um - w*U/Q -vg/D*U -Ib*V)*DT - k*(Ib*DT*DT) 
                change_sum_U += np.abs((self.tw_cell[i]['U']-U)/U)

        print('\nBeam loading study state calculation: {0:.6e} [s]\n'.format(T))
        
        # ----- セルごとの RF 電力 [W] の計算 -----
        for i in range(1, self.N_cell+1):
            vg = self.tw_cell[i]['vg']*c
            U = self.tw_cell[i]['U']
            L = self.tw_cell[i]['D']*1e-3            
            self.tw_cell[i]['RF'] = vg*U/L



    # -------------------------------------------------------------------
    # GPTインプット用の定在波の部分(カップラーの半分)を計算する
    # -------------------------------------------------------------------
    def set_sw_para(self):
        ''' GPTインプット用の定在波の部分(カップラーの半分)を計算する '''

        deg = np.pi/180.0;

        #----------- input region ----------
        L  = self.sw_cell['in']['p_len'] + self.sw_cell['in']['D_half']
        self.sw_cell['in']['zs'] = self.tw_cell[1]['zs'] - L
        self.sw_cell['in']['ze'] = self.tw_cell[1]['zs']
                       
        Ez_SS_S = self.tw_cell[1]['ampSS']*self.tw_cell[1]['Ez_S_SS']  # TW Cell の左端のEz
        Ez_OO_S = self.tw_cell[1]['ampOO']*self.tw_cell[1]['Ez_S_OO']

        phase = self.tw_cell[1]['phase']
        Ez_in_TW1_abs = sqrt(Ez_SS_S**2 + Ez_OO_S**2)
        Ez_in_SW1_abs = np.abs(self.sw_cell['in']['Ez_E'])
        Ez_zero_TW1 = Ez_SS_S*cos(phase*deg)+Ez_OO_S*cos((phase+self.dp_ri)*deg)
        Ez_zero_SW1 = self.sw_cell['in']['Ez_E']*cos(phase*deg)
        sign = -1 if Ez_zero_TW1*Ez_zero_SW1 <= 0 else 1
        amp = sign*Ez_in_TW1_abs/Ez_in_SW1_abs;

        self.sw_cell['in']['phase'] = phase
        self.sw_cell['in']['amp']   = amp

        #----------- output region ----------
        L  = self.sw_cell['out']['p_len'] + self.sw_cell['out']['D_half']
        self.sw_cell['out']['zs'] = self.tw_cell[self.N_cell]['ze']
        self.sw_cell['out']['ze'] = self.tw_cell[self.N_cell]['ze'] + L

        Ez_SS_E = self.tw_cell[self.N_cell]['ampSS']*\
            self.tw_cell[self.N_cell]['Ez_E_SS'] # TW Cell の左端のEz
        Ez_OO_E = self.tw_cell[self.N_cell]['ampOO']*\
            self.tw_cell[self.N_cell]['Ez_E_OO']
        
        Ez_in_TW2_abs = sqrt(Ez_SS_E**2 + Ez_OO_E**2)
        Ez_in_SW2_abs = np.abs(self.sw_cell['out']['Ez_S'])
        phase = self.tw_cell[self.N_cell]['phase']
        Ez_zero_TW2 = Ez_SS_E*cos(phase*deg)+Ez_OO_E*cos((phase+self.dp_ri)*deg)
        Ez_zero_SW2 = self.sw_cell['out']['Ez_E']*cos((phase+self.phase_shift)*deg)
        sign = -1 if Ez_zero_TW2*Ez_zero_SW2 else 1
        amp = sign*Ez_in_TW2_abs/Ez_in_SW2_abs;

        self.sw_cell['out']['phase'] = phase + self.phase_shift
        self.sw_cell['out']['amp']   = amp


    # -------------------------------------------------------------------
    # GPTインプット用の進行波部分のファイル (*.in) を作成する
    # -------------------------------------------------------------------
    def write_gpt_input_TW(self):
        ''' GPTインプット用の進行波部分のファイル (*.in) を作成する '''

        f = open(self.gpt_in_path, 'a')
        for i in range(1, self.N_cell+1):
            f.write('#----- cell #{0:02d} -------\n'.format(i))

            #------------- Short-Short field ---------------
            f.write('map25D_TM("wcs", "z", ')
            f.write('(offset_z{0:+.3f})*mm, '.format(self.tw_cell[i]['zs']))
            f.write('"{0:s}/{1:s}", '.format(self.GPT['path'], self.tw_cell[i]['gdf_SS']))
            f.write('"R", "Z", "Er", "Ez", "H", ')
            f.write('{0:e}, '.format(self.tw_cell[i]['ampSS']))
            f.write('0, ')
            f.write('(offset_p{0:+.2f})*degree, '.format(self.tw_cell[i]['phase']))
            f.write('omega);\n')
	
	    #------------- Open-Open field ---------------
            f.write('map25D_TM("wcs", "z", ')
            f.write('(offset_z{0:+.3f})*mm, '.format(self.tw_cell[i]['zs']))
            f.write('"{0:s}/{1:s}", '.format(self.GPT['path'], self.tw_cell[i]['gdf_OO']))
            f.write('"R", "Z", "Er", "Ez", "H", ')
            f.write('{0:e}, '.format(self.tw_cell[i]['ampOO']))
            f.write('0, ')
            f.write('(offset_p{0:+.2f})*degree, '.format(self.tw_cell[i]['phase']+self.dp_ri))
            f.write('omega);\n')

            #------------- center disk ---------------
            zc = (self.tw_cell[i]['zs'] + self.tw_cell[i]['ze'])/2.0
            R  = self.tw_cell[i]['2a']/2.0
            L  = self.tw_cell[i]['t']
            f.write('rmax("wcs", "z", ')
            f.write('(offset_z{0:+.3f})*mm, '.format(zc))
            f.write('{0:.3f}*mm, '.format(R))
            f.write('{0:.3f}*mm'.format(L))
            f.write(');\n')

        f.close()

    # -------------------------------------------------------------------
    # GPTインプット用の定在波部分のファイル (*.in) を作成する
    # -------------------------------------------------------------------
    def write_gpt_input_SW(self, cpl):
        ''' GPTインプット用の定在波部分のファイル (*.in) を作成する '''

        f = open(self.gpt_in_path, 'a')

        f.write('#----- SW region ({0:s}) -------\n'.format(cpl))
        f.write('map25D_TM("wcs", "z", ')
        f.write('(offset_z{0:+.3f})*mm, '.format(self.sw_cell[cpl]['zs']))
        f.write('"{0:s}/{1:s}", '.format(self.GPT['path'], self.sw_cell[cpl]['gdf']))
        f.write('"R", "Z", "Er", "Ez", "H", ')
        f.write('{0:e}, '.format(self.sw_cell[cpl]['amp']))
        f.write('0, ')
        f.write('(offset_p{0:+.2f})*degree, '.format(self.sw_cell[cpl]['phase']))
        f.write('omega);\n')

        #------------- input/output pipe ---------------
        if cpl == 'in':
            zc = self.sw_cell[cpl]['zs'] + self.sw_cell[cpl]['p_len']/2.0
        elif cpl == "out":
            zc = self.sw_cell[cpl]['ze'] - self.sw_cell[cpl]['p_len']/2.0
        else:
            print('SW region type error !!\n')
            print('cpl: {0:s\n'.format(cpl))
            exit()

        f.write('rmax("wcs", "z", ')
        f.write('(offset_z{0:+.3f})*mm, '.format(zc))
        f.write('{0:.3f}*mm, '.format(self.sw_cell[cpl]['p_dia']/2.0))
        f.write('{0:.3f}*mm'.format(self.sw_cell[cpl]['p_len']))
        f.write(');\n')
        f.close()


    # -------------------------------------------------------------------
    # セルの位置と位相情報を記述する
    # -------------------------------------------------------------------
    def mk_cell_z_phase_file(self):
        ''' mk_phase_plot で使うセルの位相情報を記載したファイルの作成． '''

        file_path = os.path.join(self.GPT_dir, 'cell_z_phase.txt')

        f = open(file_path, 'w')
        f.write('#'+60*'='+'\n')
        f.write('#zs\tze\tphase\n')
        f.write('#[mm]\t[mm]\t[deg]\n')
        f.write('#'+60*'='+'\n')
        # --- input sw region ---
        f.write('{0:.6f}\t{1:.6f}\t{2:.6f}\n'.\
                format(self.sw_cell['in']['zs'],\
                       self.sw_cell['in']['ze'],\
                       self.sw_cell['in']['phase']))

        # ---- TW region ---
        for i in range(1, self.N_cell+1):
            f.write('{0:.6f}\t{1:.6f}\t{2:.6f}\n'.\
                    format(self.tw_cell[i]['zs'],\
                           self.tw_cell[i]['ze'],\
                           self.tw_cell[i]['phase']))

        # --- input sw region ---
        f.write('{0:.6f}\t{1:.6f}\t{2:.6f}\n'.\
                format(self.sw_cell['out']['zs'],\
                       self.sw_cell['out']['ze'],\
                       self.sw_cell['out']['phase']))
            
        f.close()
