# -*- coding: utf-8 -*-
#!/usr/bin/python3

import re
import configparser
import os.path
import numpy as np
import scipy.constants as physics


###################################################################################
#
#  計算用のインプットファイルを読み込むためのクラス
#
###################################################################################
class Input(object):
    # ----- インプットファイルのデータの辞書の定義 (セクション，オプション)  ---------
    def_data_format = {
        'conditions':['title', 'dx', 'freq', 'is_disp'],\
        'cell': ['Number_List[]'],\
        'GPT': ['path', 'mesh_size', 'RF_PWR', 'Ib', 'Qfac',\
                'StrZ', 'InitP',\
                'in_cpl', 'out_cpl']\
        }
    input_file_dir = '../input_data'

        
    #-------------------------------------------------------------------
    # SUPERFISH で自動計算する加速管の定義ファイルの読込み
    #-------------------------------------------------------------------
    def __init__(self, input_file):
        ''' 初期化を行います．'''

        input_file = Input.input_file_dir+'/'+input_file
        
        # --- 入力ファイルのチェックと設定 ---
        print("Input file: ", input_file)
        if not(os.path.isfile(input_file)):
            print('\nThere is no input file !!!')
            print('\tinput file: {0:s}'.format(input_file))
            exit()
        input_abs_dir   = os.path.dirname(os.path.abspath(input_file))

        (file_name, file_ext) = os.path.splitext(input_file)

        # --- ファイルの読み込み ---
        self.rdata = configparser.ConfigParser(inline_comment_prefixes=('#', ';'), strict=False)
        self.rdata.optionxform = str                   # キーの大文字/小文字を区別
        self.rdata.read(input_file, encoding='utf8')   # データの読み込み
        
        
        # --- 読み込んだデータのチェック ---
        self.set_def_data_format()
        self.check_sections()
        self.check_options()
        self.check_array_options('cell')

        # --- 読み込んだデータを格納する辞書の初期化 ---
        self.input     = {'file':input_file, 'abs_dir':input_abs_dir,
                          'file_name':file_name, 'ext':file_ext}
        self.conditions = {}
        self.cell       = {}
        self.GPT        = {}
    
        # --- 読み込んだデータの辞書への格納 ---
        self.set_data('conditions', self.conditions)
        self.set_data('cell', self.cell)
        if self.conditions['is_GPT']: self.set_data('GPT', self.GPT)

        # --- ひとつの辞書にする ---
        if self.conditions['is_GPT']:
            self.acc = {'conditions': self.conditions,\
                        'cell': self.cell,\
                        'GPT': self.GPT}
        else:
            self.acc = {'conditions': self.conditions,\
                        'cell': self.cell}

        
    #-------------------------------------------------------------------
    # データフォーマットの定義辞書(def_data_format)の設定
    #-------------------------------------------------------------------
    def set_def_data_format(self):
        ''' データフォーマットの定義辞書(def_data_format)を設定する '''

        if not 'GPT' in self.rdata.sections():
            Input.def_data_format.pop('GPT')

    #-------------------------------------------------------------------
    # インプットデータのセクションのチェック
    #-------------------------------------------------------------------
    def check_sections(self):
        ''' インプットデータのセクションのチェックを行います '''
        defi_key = set(k for k in Input.def_data_format.keys())    
        read_key = set(self.rdata.sections())

        if (len(defi_key - read_key) != 0 ) or \
                (len(read_key - defi_key) != 0 ):
            print("\nerror occured at check_sections in READ_INPUT.py !!!!")
            print("\tdiff: ", defi_key)
            print("\tread: ", read_key)
            print("diff - read:\t", defi_key - read_key)
            print("read - diff:\t", read_key - defi_key)
            exit()

    #-------------------------------------------------------------------
    # インプットデータのオプションのチェック
    #-------------------------------------------------------------------
    def check_options(self):
        ''' インプットデータのオプションのチェック '''
        num_test = re.compile(r'Number_List\[\]', flags=0)
        for section in Input.def_data_format.keys():       # データ定義辞書からセクション
            option = Input.def_data_format[section]        # オプションのリスト
            if re.match(num_test, option[0]):
                self.check_array_options(section)
            else:
                defi = set(Input.def_data_format[section])
                read = self.option_set(section)

                if (len(defi - read) != 0 ) or  (len(read - defi) != 0 ):
                    print("\nerror occured at check_options in READ_INPUT.py !!!!")
                    print('\tsection: ', section)
                    print("\tdiff: ", defi)
                    print('\tread: ', read)
                    print('\tdiff - read: ', defi - read)
                    print('\tread - diff: ', read - defi)
                    exit()

    #-------------------------------------------------------------------
    # インプットデータの配列型オプションのチェック
    #-------------------------------------------------------------------
    def check_array_options(self, section):
        ''' インプットデータの配列型オプションのチェック '''
        num = re.compile(r'\d+', flags=0)
        count  = 0
        nm = []
        for x in self.rdata.options(section):
            if re.match(num, x):
                count  += 1
                nm.append(int(x))
        alist = [i for i in range(1, count+1)]
        rset  = set(nm)
        cset  = set(alist)

        if rset != cset:
            print('\n\n入力エラー  [{0:s}]'.format(section))
            print('\t入力配列番号: ', rset)
            print('\t問題の配列番号: ',rset-cset, cset-rset)
            print('\n\n')
            exit()
        
            
    #-------------------------------------------------------------------
    # オプションの集合を返す
    #-------------------------------------------------------------------
    def option_set(self, section):
        ''' オプションの集合を返す．'''
        ary = re.compile('[a-zZ-Z]\w*\[\d+\]', flags=0)
        num = re.compile('\[\d+\]')
        op_set = set()
        for op in self.rdata.options(section):
            if re.match(ary, op):
                op_set.add(re.sub(num, '[]',op))
                continue
            op_set.add(op)
            
        return op_set
        
                
    #-------------------------------------------------------------------
    # データの設定
    #-------------------------------------------------------------------
    def set_data(self, section, dic):
        ''' 読み込んだデータを数値データとして，辞書に格納する． '''
        def error(s, k, d):
            print('\n\nError occured at ReadData.py > set_data()')
            print('section: {0:s}\tkey:{1:s}\tdata:{2:s}\n\n'.format(
                s, k, d))
            exit()

        str_data = (('conditions', 'title'), ('GPT', 'path'))
        cpl_data = (('GPT', 'in_cpl'), ('GPT', 'out_cpl'))
        for key in self.rdata[section]:
            if (section, key) in str_data:
               dic[key] = self.rdata[section][key]
            elif section == 'conditions' and key == 'is_disp':
                dat = self.rdata[section][key].lower()
                if dat == 'true':
                    dic[key] = True
                elif dat == 'false':
                    dic[key] = False
                else:
                    error(section, key, self.rdata[section][key])
            elif section == 'cell':
                key_no = int(key)
                d = re.split(r'[,\s]+', self.rdata[section][key])
                if len(d) != 3:
                    print(" [cell] > cell[] のデータの数が過不足です．")
                    print('input: ', d)
                    exit()
                dic[key_no] = {'beta':float(d[0]), \
                               '2a':float(d[1]), \
                               't':float(d[2])}
            elif (section, key) in cpl_data:
                d = re.split(r'[,\s]+', self.rdata[section][key])
                if len(d) != 2:
                    print('[GPT] > {0:s} のデータの数が過不足です．'.format(key))
                    print('input: ', d)
                    exit()
                dic[key] = {'p_len':float(d[0]), \
                            'p_dia':float(d[1])}
            else:
                dic[key] = self.rdata.getfloat(section, key)

        if section == 'conditions':
            if 'GPT' in self.rdata.sections():
                dic['is_GPT'] = True
            else:
                dic['is_GPT'] = False

    #-------------------------------------------------------------------
    # 配列名を[]付きで返す．変数の場合は変数名のみ
    #-------------------------------------------------------------------
    def array_name(self, key):
        ''' オプションの名前を返す．'''
        ary = re.compile('[a-zZ-Z]\w*\[\d+\]', flags=0)
        num = re.compile('\[\d+\]')
        if re.match(ary, key):
            return re.sub(num, '[]', key)
        else:
            return key 
    

    #-------------------------------------------------------------------
    # セクションのリストを返す
    #-------------------------------------------------------------------
    def get_sections(self):
        ''' セクションのリストを返す '''
        return self.rdata.sections() 


    #-------------------------------------------------------------------
    # インプットファイルのセクションとキー，値を表示する
    #-------------------------------------------------------------------
    def print_input_raw_data(self):
        for section in self.rdata.sections():
            print('\n[{0:s}]'.format(section))
            for key in self.rdata.options(section):
                print('\t{0:s}:\t{1:s}'.format(key, self.rdata[section][key]))

                
    #-------------------------------------------------------------------
    # インプットファイルのセクションとキー，値を表示する
    #-------------------------------------------------------------------
    def print_data(self):
        for section in self.Input.keys():
            print('\n[{0:s}]'.format(section))
            for key in self.Input[section].keys():
                val = self.Input[section][key]
                if type(val) is float:
                    if 1e-3 < np.abs(val) and np.abs(val) < 1e5:
                        print('\t{0:<10.10s}\t{1: >10.5f}'.format(key, val))
                    else:
                        print('\t{0:<10.10s}\t{1: >10.5e}'.format(key, val))
                elif type(val) is int:
                    print('\t{0:<10.10s}\t{1: >4d}'.format(key, val))
                elif type(val) is bool:
                    print('\t{0:<10.10s}\t{1:<10s}'.format(key, str(val)))
                elif type(val) is str:
                    print('\t{0:<10.10s}\t{1:<10s}'.format(key, val))
                elif type(val) is list:
                    print('\t{0:<10.10s}'.format(key))
                    for v in val:
                        print('\t\t\t', v)
                else:
                    print('データの型が不明\n')
                    exit()
        print('\n')

    #-------------------------------------------------------------------
    # インプットデータを返す
    #-------------------------------------------------------------------
    def get_input_datas(self):
        return self.acc

